@@ -1,32 +1,17 @@
 # -*- coding: utf-8 -*-
 
-# MIT license
-#
-# Copyright (C) 2018 by XESS Corp.
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
 
 """
 Handles complete circuits made of parts and nets.
 """
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import (  # isort:skip
+    absolute_import,
+    division,
+    print_function,
+    unicode_literals,
+)
 
 import functools
 import json
@@ -41,11 +26,10 @@ from future import standard_library
 
 from .arrange import Arranger
 from .bus import Bus
-from .common import *
-from .defines import *
+from .common import builtins
 from .erc import dflt_circuit_erc
 from .interface import Interface
-from .logger import erc_logger, logger
+from .logger import active_logger, erc_logger
 from .net import NCNet, Net
 from .part import Part, PartUnit
 from .pckg_info import __version__
@@ -132,7 +116,7 @@ class Circuit(SkidlBaseObject):
 
     def __enter__(self):
         """Create a context for making this circuit the default_circuit."""
-        self.circuit_stack.append(builtins.default_circuit)
+        self.circuit_stack.append(default_circuit)
         builtins.default_circuit = self
         return self
 
@@ -142,8 +126,7 @@ class Circuit(SkidlBaseObject):
     def add_hierarchical_name(self, name):
         """Record a new hierarchical name.  Throw an error if it is a duplicate."""
         if name in self._hierarchical_names:
-            log_and_raise(
-                logger,
+            active_loggerraise_(
                 ValueError,
                 "Can't add duplicate hierarchical name {} to this circuit.".format(
                     name
@@ -156,8 +139,7 @@ class Circuit(SkidlBaseObject):
         try:
             self._hierarchical_names.remove(name)
         except KeyError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't remove non-existent hierarchical name {} from circuit.".format(
                     name
@@ -178,21 +160,20 @@ class Circuit(SkidlBaseObject):
 
                     # Add the part to this circuit.
                     part.circuit = self  # Record the Circuit object for this part.
-                    part.ref = part.ref  # This adjusts the part reference if necessary.
+                    part.ref = part.ref  # Adjusts the part reference if necessary.
 
-                    part.hierarchy = self.hierarchy  # Store hierarchy of part.
+                    # Store hierarchy of part.
+                    part.hierarchy = self.hierarchy
 
                     # Check the part does not have a conflicting hierarchical name
                     self.add_hierarchical_name(part.hierarchical_name)
 
-                    part.skidl_trace = (
-                        get_skidl_trace()
-                    )  # Store part instantiation trace.
+                    # Store part instantiation trace.
+                    part.skidl_trace = ";".join(get_skidl_trace())
 
                     self.parts.append(part)
                 else:
-                    log_and_raise(
-                        logger,
+                    active_logger.raise_(
                         ValueError,
                         "Can't add unmovable part {} to this circuit.".format(part.ref),
                     )
@@ -207,14 +188,13 @@ class Circuit(SkidlBaseObject):
                     part.hierarchy = None
                     self.parts.remove(part)
                 else:
-                    logger.warning(
+                    active_logger.warning(
                         "Removing non-existent part {} from this circuit.".format(
                             part.ref
                         )
                     )
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't remove part {} from this circuit.".format(part.ref),
                 )
@@ -239,8 +219,7 @@ class Circuit(SkidlBaseObject):
                     self.nets.append(net)
 
                 else:
-                    log_and_raise(
-                        logger,
+                    active_logger.raise_(
                         ValueError,
                         "Can't add unmovable net {} to this circuit.".format(net.name),
                     )
@@ -254,14 +233,13 @@ class Circuit(SkidlBaseObject):
                     net.hierarchy = None
                     self.nets.remove(net)
                 else:
-                    logger.warning(
+                    active_logger.warning(
                         "Removing non-existent net {} from this circuit.".format(
                             net.name
                         )
                     )
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't remove unmovable net {} from this circuit.".format(net.name),
                 )
@@ -299,14 +277,13 @@ class Circuit(SkidlBaseObject):
                     for net in bus.nets:
                         self -= net
                 else:
-                    logger.warning(
+                    active_logger.warning(
                         "Removing non-existent bus {} from this circuit.".format(
                             bus.name
                         )
                     )
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't remove unmovable bus {} from this circuit.".format(bus.name),
                 )
@@ -326,8 +303,7 @@ class Circuit(SkidlBaseObject):
                         except AttributeError:
                             pass
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't add the same package to more than one circuit.",
                 )
@@ -345,14 +321,13 @@ class Circuit(SkidlBaseObject):
                         except AttributeError:
                             pass
                 else:
-                    logger.warning(
+                    active_logger.active_logger.warning(
                         "Removing non-existent package {} from this circuit.".format(
                             package.name
                         )
                     )
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't remove unmovable package {} from this circuit.".format(
                         package.name
@@ -374,8 +349,7 @@ class Circuit(SkidlBaseObject):
             elif isinstance(thing, Package):
                 self.add_packages(thing)
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't add a {} to a Circuit object.".format(type(thing)),
                 )
@@ -396,10 +370,9 @@ class Circuit(SkidlBaseObject):
             elif isinstance(thing, Package):
                 self.rmv_packages(thing)
             else:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
-                    "Can't remove a {} from a Circuit object.".format(type(pnb)),
+                    "Can't remove a {} from a Circuit object.".format(type(thing)),
                 )
         return self
 
@@ -468,26 +441,24 @@ class Circuit(SkidlBaseObject):
     def ERC(self, *args, **kwargs):
         """Run class-wide and local ERC functions on this circuit."""
 
+        # Save the currently active logger and activate the ERC logger.
+        active_logger.push(erc_logger)
+
         # Reset the counters to clear any warnings/errors from previous ERC run.
-        erc_logger.error.reset()
-        erc_logger.warning.reset()
+        active_logger.error.reset()
+        active_logger.warning.reset()
 
         self._preprocess()
 
         if self.no_files:
-            erc_logger.stop_file_output()
+            active_logger.stop_file_output()
 
         super().ERC(*args, **kwargs)
 
-        if (erc_logger.error.count, erc_logger.warning.count) == (0, 0):
-            sys.stderr.write("\nNo ERC errors or warnings found.\n\n")
-        else:
-            sys.stderr.write(
-                "\n{} warnings found during ERC.\n".format(erc_logger.warning.count)
-            )
-            sys.stderr.write(
-                "{} errors found during ERC.\n\n".format(erc_logger.error.count)
-            )
+        active_logger.report_summary("running ERC")
+
+        # Restore the logger that was active before the ERC.
+        active_logger.pop()
 
     def generate_netlist(self, **kwargs):
         """
@@ -506,8 +477,8 @@ class Circuit(SkidlBaseObject):
         from . import skidl
 
         # Reset the counters to clear any warnings/errors from previous run.
-        logger.error.reset()
-        logger.warning.reset()
+        active_logger.error.reset()
+        active_logger.warning.reset()
 
         self._preprocess()
 
@@ -523,29 +494,14 @@ class Circuit(SkidlBaseObject):
             gen_func = getattr(self, "_gen_netlist_{}".format(tool))
             netlist = gen_func(**kwargs)  # Pass any remaining arguments.
         except KeyError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't generate netlist in an unknown ECAD tool format ({}).".format(
                     tool
                 ),
             )
 
-        if (logger.error.count, logger.warning.count) == (0, 0):
-            sys.stderr.write(
-                "\nNo errors or warnings found during netlist generation.\n\n"
-            )
-        else:
-            sys.stderr.write(
-                "\n{} warnings found during netlist generation.\n".format(
-                    logger.warning.count
-                )
-            )
-            sys.stderr.write(
-                "{} errors found during netlist generation.\n\n".format(
-                    logger.error.count
-                )
-            )
+        active_logger.report_summary("generating netlist")
 
         if not self.no_files:
             with opened(file_ or (get_script_name() + ".net"), "w") as f:
@@ -558,6 +514,55 @@ class Circuit(SkidlBaseObject):
 
         return netlist
 
+    def generate_pcb(self, **kwargs):
+        """
+        Create a PCB file from the circuit.
+
+        Args:
+            file_: Either a file object that can be written to, or a string
+                containing a file name, or None.
+            tool: The EDA tool the netlist will be generated for.
+            do_backup: If true, create a library with all the parts in the circuit.
+
+        Returns:
+            None.
+        """
+
+        from . import skidl
+
+        # Reset the counters to clear any warnings/errors from previous run.
+        active_logger.error.reset()
+        active_logger.warning.reset()
+
+        self._preprocess()
+
+        # Extract arguments:
+        #     Get EDA tool the netlist will be generated for.
+        #     Get file the netlist will be stored in (if any).
+        #     Get flag controlling the generation of a backup library.
+        tool = kwargs.pop("tool", skidl.get_default_tool())
+        file_ = kwargs.pop("file_", None)
+        do_backup = kwargs.pop("do_backup", True)
+
+        if not self.no_files:
+            try:
+                gen_func = getattr(self, "_gen_pcb_{}".format(tool))
+            except KeyError:
+                active_logger.raise_(
+                    ValueError,
+                    "Can't generate PCB in an unknown ECAD tool format ({}).".format(
+                        tool
+                    ),
+                )
+            else:
+                if do_backup:
+                    self.backup_parts()  # Create a new backup lib for the circuit parts.
+                    global backup_lib  # Clear out any old backup lib so the new one
+                    backup_lib = None  #   will get reloaded when it's needed.
+                gen_func(file_)  # Generate the PCB file from the netlist.
+
+        active_logger.report_summary("creating PCB")
+
     def generate_xml(self, file_=None, tool=None):
         """
         Return netlist as an XML string and also write it to a file/stream.
@@ -573,8 +578,8 @@ class Circuit(SkidlBaseObject):
         from . import skidl
 
         # Reset the counters to clear any warnings/errors from previous run.
-        logger.error.reset()
-        logger.warning.reset()
+        active_logger.error.reset()
+        active_logger.warning.reset()
 
         self._preprocess()
 
@@ -585,23 +590,12 @@ class Circuit(SkidlBaseObject):
             gen_func = getattr(self, "_gen_xml_{}".format(tool))
             netlist = gen_func()
         except KeyError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't generate XML in an unknown ECAD tool format ({}).".format(tool),
             )
 
-        if (logger.error.count, logger.warning.count) == (0, 0):
-            sys.stderr.write("\nNo errors or warnings found during XML generation.\n\n")
-        else:
-            sys.stderr.write(
-                "\n{} warnings found during XML generation.\n".format(
-                    logger.warning.count
-                )
-            )
-            sys.stderr.write(
-                "{} errors found during XML generation.\n\n".format(logger.error.count)
-            )
+        active_logger.report_summary("generating XML")
 
         if not self.no_files:
             with opened(file_ or (get_script_name() + ".xml"), "w") as f:
@@ -783,16 +777,20 @@ class Circuit(SkidlBaseObject):
         from . import skidl
 
         # Reset the counters to clear any warnings/errors from previous run.
-        logger.error.reset()
-        logger.warning.reset()
+        active_logger.error.reset()
+        active_logger.warning.reset()
 
         self._preprocess()
 
         # Get the list of nets which will be routed and not represented by stubs.
         # Search all nets for those set as stubs or that are no-connects.
-        net_stubs = [n for n in self.nets if getattr(n, 'stub', False) or isinstance(n, NCNet)]
+        net_stubs = [
+            n for n in self.nets if getattr(n, "stub", False) or isinstance(n, NCNet)
+        ]
         # Also find buses that are set as stubs and add their individual nets.
-        net_stubs.extend(expand_buses([b for b in self.buses if getattr(b, 'stub', False)]))
+        net_stubs.extend(
+            expand_buses([b for b in self.buses if getattr(b, "stub", False)])
+        )
         routed_nets = list(set(self.nets) - set(net_stubs))
 
         # Assign each routed net a unique integer. Interconnected nets
@@ -814,7 +812,9 @@ class Circuit(SkidlBaseObject):
                     io = io_dict[net.netio.lower()[0]]
                     ports[net.name] = {
                         "direction": io,
-                        "bits": [net_nums[net.name],],
+                        "bits": [
+                            net_nums[net.name],
+                        ],
                     }
                 except AttributeError:
                     # Net has no netio so don't assign a port.
@@ -859,7 +859,9 @@ class Circuit(SkidlBaseObject):
 
                 # Associate each connected pin of a part with the assigned net number.
                 connections = {
-                    pin.num: [net_nums[pin.net.name],]
+                    pin.num: [
+                        net_nums[pin.net.name],
+                    ]
                     for pin in pins
                     if pin.net in routed_nets
                 }
@@ -889,29 +891,39 @@ class Circuit(SkidlBaseObject):
                     "type": name,
                     "port_directions": part_pin_dirs,
                     "connections": connections,
-                    "attributes": {"value": str(part.value),},
+                    "attributes": {
+                        "value": str(part.value),
+                    },
                 }
 
-        schematic_json = {"modules": {self.name: {"ports": ports, "cells": cells,}}}
+        schematic_json = {
+            "modules": {
+                self.name: {
+                    "ports": ports,
+                    "cells": cells,
+                }
+            }
+        }
 
         if not self.no_files:
             file_basename = file_ or get_script_name()
             json_file = file_basename + ".json"
             svg_file = file_basename + ".svg"
+
             with opened(json_file, "w") as f:
                 f.write(
                     json.dumps(
                         schematic_json, sort_keys=True, indent=2, separators=(",", ": ")
                     )
                 )
+
             skin_file = file_basename + "_skin.svg"
             with opened(skin_file, "w") as f:
                 f.write(self.generate_netlistsvg_skin(net_stubs=net_stubs))
-            subprocess.call(
-                "netlistsvg {json_file} --skin {skin_file} -o {svg_file}".format(
-                    **locals()
-                ),
-                shell=True,
+
+            subprocess.Popen(
+                ["netlistsvg", json_file, "--skin", skin_file, "-o", svg_file],
+                shell=False,
             )
 
         return schematic_json
@@ -931,8 +943,8 @@ class Circuit(SkidlBaseObject):
         from . import skidl
 
         # Reset the counters to clear any warnings/errors from previous run.
-        logger.error.reset()
-        logger.warning.reset()
+        active_logger.active_logger.active_logger.error.reset()
+        active_logger.warning.reset()
 
         self._preprocess()
 
@@ -955,29 +967,14 @@ class Circuit(SkidlBaseObject):
                 gen_func = getattr(self, "_gen_schematic_{}".format(tool))
                 gen_func(route)
             except KeyError:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     ValueError,
                     "Can't generate schematic in an unknown ECAD tool format ({}).".format(
                         tool
                     ),
                 )
 
-        if (logger.error.count, logger.warning.count) == (0, 0):
-            sys.stderr.write(
-                "\nNo errors or warnings found during schematic generation.\n\n"
-            )
-        else:
-            sys.stderr.write(
-                "\n{} warnings found during schematic generation.\n".format(
-                    logger.warning.count
-                )
-            )
-            sys.stderr.write(
-                "{} errors found during schematic generation.\n\n".format(
-                    logger.error.count
-                )
-            )
+        active_logger.report_summary("generating schematic")
 
     def generate_dot(
         self,
@@ -1015,8 +1012,8 @@ class Circuit(SkidlBaseObject):
         """
 
         # Reset the counters to clear any warnings/errors from previous run.
-        logger.error.reset()
-        logger.warning.reset()
+        active_logger.error.reset()
+        active_logger.warning.reset()
 
         self._preprocess()
 
@@ -1030,7 +1027,7 @@ class Circuit(SkidlBaseObject):
 
         # Add stubbed nets to split_nets:
         split_nets = split_nets[:]  # Make a local copy.
-        split_nets.extend([n.name for n in nets if getattr(n, 'stub', False)])
+        split_nets.extend([n.name for n in nets if getattr(n, "stub", False)])
 
         for i, n in enumerate(nets):
             xlabel = n.name
@@ -1091,17 +1088,21 @@ class Circuit(SkidlBaseObject):
         """
 
         from . import skidl
+        from .tools import SKIDL
+
+        if self.no_files:
+            return
 
         self._preprocess()
 
         lib = SchLib(tool=SKIDL)  # Create empty library.
         for p in self.parts:
             lib += p
+
         if not file_:
             file_ = skidl.BACKUP_LIB_FILE_NAME
 
-        if not self.no_files:
-            lib.export(libname=skidl.BACKUP_LIB_NAME, file_=file_)
+        lib.export(libname=skidl.BACKUP_LIB_NAME, file_=file_)
 
 
 __func_name_cntr = defaultdict(int)
