@@ -1,32 +1,19 @@
 # -*- coding: utf-8 -*-
 
-# MIT license
-#
-# Copyright (C) 2018 by XESS Corp.
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
+
 """
 Handles parts.
 """
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import (  # isort:skip
+    absolute_import,
+    division,
+    print_function,
+    unicode_literals,
+)
 
+import functools
 import re
 from builtins import dict, int, object, range, str, super, zip
 from copy import copy
@@ -35,9 +22,8 @@ from random import randint
 from future import standard_library
 
 from .common import *
-from .defines import *
 from .erc import dflt_part_erc
-from .logger import logger
+from .logger import active_logger
 from .skidlbaseobj import SkidlBaseObject
 from .utilities import *
 
@@ -53,6 +39,13 @@ except ImportError:
         pass
 
 
+# Places where parts can be stored.
+#   NETLIST: The part will become part of a circuit netlist.
+#   LIBRARY: The part will be placed in the part list for a library.
+#   TEMPLATE: The part will be used as a template to be copied from.
+NETLIST, LIBRARY, TEMPLATE = ["NETLIST", "LIBRARY", "TEMPLATE"]
+
+
 class PinNumberSearch(object):
     """
     A class for restricting part pin indexing to only pin numbers
@@ -104,13 +97,6 @@ class Part(SkidlBaseObject):
     """
     A class for storing a definition of a schematic part.
 
-    Attributes:
-        ref: String storing the reference of a part within a schematic (e.g., 'R5').
-        value: String storing the part value (e.g., '3K3').
-        footprint: String storing the PCB footprint associated with a part (e.g., SOIC-8).
-        pins: List of Pin objects for this part.
-        tag: A tag string to link a part to its footprint in a pcb editor.
-
     Args:
         lib: Either a SchLib object or a schematic part library file name.
         name: A string with name of the part to find in the library, or to assign to
@@ -123,6 +109,11 @@ class Part(SkidlBaseObject):
         part_defn: A list of strings that define the part (usually read from a
             schematic library file).
         circuit: The Circuit object this Part belongs to.
+        ref_prefix: Prefix for part references such as 'U' or 'J'.
+        ref: A specific part reference to be assigned.
+        tag: A specific tag to tie the part to its footprint in the PCB.
+        pin_splitters: String of characters that split long pin names into shorter aliases.
+        tool_version: Select between KiCad V5 and V6.
 
     Keyword Args:
         kwargs: Name/value pairs for setting attributes for the part.
@@ -146,11 +137,18 @@ class Part(SkidlBaseObject):
         connections=None,
         part_defn=None,
         circuit=None,
+        ref_prefix="U",
+        ref=None,
+        tag=None,
+        pin_splitters=None,
+        tool_version=None,
         **kwargs
     ):
 
         import skidl
+
         from .schlib import SchLib
+        from .tools import SKIDL
 
         super().__init__()
 
@@ -166,18 +164,11 @@ class Part(SkidlBaseObject):
         self.name = name  # Assign initial part name.
         self.description = ""  # Make sure there is a description, even if empty.
         self._ref = ""  # Provide a member for holding a reference.
-        self.ref_prefix = ""  # Provide a member for holding the part reference prefix.
+        self.ref_prefix = ref_prefix  # Store the part reference prefix.
         self.tool = tool  # Initial type of part (SKIDL, KICAD, etc.)
         self.circuit = None  # Part starts off unassociated with any circuit.
         self.match_pin_regex = False  # Don't allow regex matches of pin names.
 
-        # Remove a part reference if it has been explicitly set as None.
-        # Otherwise, this causes the assigned part reference to be incremented twice:
-        # once by Circuit.add_parts() and again by setattr().
-        ref = kwargs.pop("ref", None)
-        if ref:
-            kwargs["ref"] = ref
-
         # Create a Part from a library entry.
         if lib:
             # If the lib argument is a string, then create a library using the
@@ -188,7 +179,7 @@ class Part(SkidlBaseObject):
                     lib = SchLib(filename=libname, tool=tool)
                 except FileNotFoundError as e:
                     if skidl.QUERY_BACKUP_LIB:
-                        logger.warning(
+                        active_logger.warning(
                             'Could not load KiCad schematic library "{}", falling back to backup library.'.format(
                                 libname
                             )
@@ -219,7 +210,6 @@ class Part(SkidlBaseObject):
 
             # If given, set the tool version before parsing the part definition.
             # At this time, this is done to support differences between KiCad V5 and V6.
-            tool_version = kwargs.pop("tool_version", None)
             if tool_version:
                 self.tool_version = tool_version
 
@@ -231,22 +221,18 @@ class Part(SkidlBaseObject):
             pass
 
         else:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't make a part without a library & part name or a part definition.",
             )
 
         # Split multi-part pin names into individual pin aliases.
-        self.split_pin_names(kwargs.pop("pin_splitters", None))
+        self.split_pin_names(pin_splitters)
 
         # Setup the tag for tieing the part to a footprint in a pcb editor.
-        # Use the user specified tag if present.
-        tag = kwargs.pop("tag", None)
-        if tag is not None:
-            self.tag = tag
-        else:
-            self.tag = str(randint(0, 2 ** 64 - 1))
+        # Do this before adding the part to the circuit or an exception will occur
+        # because the part can't give its hierarchical name to the circuit.
+        self.tag = tag or str(randint(0, 2 ** 64 - 1))
 
         if dest != LIBRARY:
             if dest == NETLIST:
@@ -267,10 +253,17 @@ class Part(SkidlBaseObject):
         # Add any XSPICE I/O as pins. (This only happens with SPICE simulations.)
         self.add_xspice_io(kwargs.pop("io", []))
 
+        # Set the part reference if one was explicitly provided.
+        if ref:
+            self.ref = ref
+
         # Add any other passed-in attributes to the part.
         for k, v in list(kwargs.items()):
             setattr(self, k, v)
 
+        # If any pins were added, make sure they're associated with the part.
+        self.associate_pins()
+
     def add_xspice_io(self, io):
         """
         Add XSPICE I/O to the pins of a part.
@@ -320,10 +313,8 @@ class Part(SkidlBaseObject):
             either their reference, name, alias, or their description.
         """
 
-        from .alias import Alias
-
         if not circuit:
-            circuit = builtins.default_circuit
+            circuit = default_circuit
 
         search_params = (
             ("ref", text, True),
@@ -341,7 +332,7 @@ class Part(SkidlBaseObject):
         return list_or_scalar(parts)
 
     def _find_min_max_pins(self):
-        """ Return the minimum and maximum pin numbers for the part. """
+        """Return the minimum and maximum pin numbers for the part."""
         pin_nums = []
         try:
             for p in self.pins:
@@ -371,8 +362,7 @@ class Part(SkidlBaseObject):
         try:
             parse_func = getattr(self, "_parse_lib_part_{}".format(self.tool))
         except AttributeError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't create a part with an unknown ECAD tool file format: {}.".format(
                     self.tool
@@ -421,8 +411,8 @@ class Part(SkidlBaseObject):
                 caps = 10 * cap             # Make an array with 10 copies of it.
         """
 
-        from .defines import NETLIST
         from .circuit import Circuit
+        from .part import NETLIST
         from .pin import Pin
 
         # If the number of copies is None, then a single copy will be made
@@ -436,16 +426,14 @@ class Part(SkidlBaseObject):
 
         # Check that a valid number of copies is requested.
         if not isinstance(num_copies, int):
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't make a non-integer number ({}) of copies of a part!".format(
                     num_copies
                 ),
             )
         if num_copies < 0:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't make a negative number ({}) of copies of a part!".format(
                     num_copies
@@ -515,7 +503,7 @@ class Part(SkidlBaseObject):
                 # Place the copied part in the explicitly-stated circuit,
                 # or the same circuit as the original,
                 # or else into the default circuit.
-                circuit = circuit or self.circuit or builtins.default_circuit
+                circuit = circuit or self.circuit or default_circuit
                 circuit += cpy
 
             # Add any XSPICE I/O as pins to the part.
@@ -527,8 +515,7 @@ class Part(SkidlBaseObject):
                     try:
                         v = v[i]
                     except IndexError:
-                        log_and_raise(
-                            logger,
+                        active_logger.raise_(
                             ValueError,
                             "{} copies of part {} were requested, but too few elements in attribute {}!".format(
                                 num_copies, self.name, k
@@ -545,7 +532,7 @@ class Part(SkidlBaseObject):
         return copies[0]
 
     def validate(self):
-        """ Check that pins and units reference the correct part that owns them. """
+        """Check that pins and units reference the correct part that owns them."""
         for pin in self.pins:
             assert pin.part == self
         for unit in self.unit.values():
@@ -562,7 +549,7 @@ class Part(SkidlBaseObject):
     __rmul__ = __mul__
 
     def copy_units(self, src):
-        """ Make copies of the units from the source part. """
+        """Make copies of the units from the source part."""
         self.unit = {}  # Remove references to any existing units.
         for label, unit in src.unit.items():
             # Get the pin numbers from the unit in the source part.
@@ -605,7 +592,12 @@ class Part(SkidlBaseObject):
             if i1 and i2:
                 break
         if i1 and i2:
-            pins[i1].num, pins[i1].name, pins[i2].num, pins[i2].name = pins[i2].num, pins[i2].name, pins[i1].num, pins[i1].name
+            pins[i1].num, pins[i1].name, pins[i2].num, pins[i2].name = (
+                pins[i2].num,
+                pins[i2].name,
+                pins[i1].num,
+                pins[i1].name,
+            )
 
     def rename_pin(self, pin_id, new_pin_name):
         """Assign a new name to a pin of a part."""
@@ -615,7 +607,7 @@ class Part(SkidlBaseObject):
                 return
 
     def renumber_pin(self, pin_id, new_pin_num):
-        "Assign a new number to a pin of a part."""
+        "Assign a new number to a pin of a part." ""
         for pin in self.pins:
             if pin_id in (pin.num, pin.name):
                 pin.num = new_pin_num
@@ -648,8 +640,8 @@ class Part(SkidlBaseObject):
                 net += atmega['RESET']  # Connects reset pin to the net.
         """
 
-        from .netpinlist import NetPinList
         from .alias import Alias
+        from .netpinlist import NetPinList
 
         # Extract restrictions on searching for only pin names or numbers.
         only_search_numbers = criteria.pop("only_search_numbers", False)
@@ -728,7 +720,7 @@ class Part(SkidlBaseObject):
     # Get pins from a part using brackets, e.g. [1,5:9,'A[0-9]+'].
     __getitem__ = get_pins
 
-    def __setitem__(self, ids, *pins_nets_buses):
+    def __setitem__(self, ids, pins_nets_buses):
         """
         You can't assign to the pins of parts. You must use the += operator.
 
@@ -751,13 +743,13 @@ class Part(SkidlBaseObject):
 
         # If the iadd_flag is set, then it's OK that we got
         # here and don't issue an error. Also, delete the flag.
-        if getattr(pins_nets_buses[0], "iadd_flag", False):
-            del pins_nets_buses[0].iadd_flag
+        if from_iadd(pins_nets_buses):
+            rmv_iadd(pins_nets_buses)
             return
 
         # No iadd_flag or it wasn't set. This means a direct assignment
         # was made to the pin, which is not allowed.
-        log_and_raise(logger, TypeError, "Can't assign to a part! Use the += operator.")
+        active_logger.raise_(TypeError, "Can't assign to a part! Use the += operator.")
 
     def __iter__(self):
         """
@@ -845,7 +837,7 @@ class Part(SkidlBaseObject):
             add_unique_attr(self, alias, pin)
         else:
             # Error: either 0 or multiple pins were found.
-            log_and_raise(logger, ValueError, "Cannot set alias for {}".format(pin_ids))
+            active_logger.raise_(ValueError, "Cannot set alias for {}".format(pin_ids))
 
     def pin_aliases_to_attributes(self):
         """Make each pin alias into an attribute of the part."""
@@ -888,7 +880,7 @@ class Part(SkidlBaseObject):
         # Warn if the unit label collides with any of the part's pin names.
         collisions = self.get_pins("^" + label + "$")  # Look for exact match.
         if collisions:
-            logger.warning(
+            active_logger.warning(
                 "Using a label ({}) for a unit of {} that matches one or more of it's pin names ({})!".format(
                     label, self.erc_desc(), collisions
                 )
@@ -1005,8 +997,7 @@ class Part(SkidlBaseObject):
         try:
             gen_func = getattr(self, "_gen_netlist_comp_{}".format(tool))
         except AttributeError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't generate netlist in an unknown ECAD tool format ({}).".format(
                     tool
@@ -1034,8 +1025,7 @@ class Part(SkidlBaseObject):
         try:
             gen_func = getattr(self, "_gen_xml_comp_{}".format(tool))
         except AttributeError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't generate XML in an unknown ECAD tool format ({}).".format(tool),
             )
@@ -1055,8 +1045,7 @@ class Part(SkidlBaseObject):
         try:
             gen_func = getattr(self, "_gen_svg_comp_{}".format(tool))
         except AttributeError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't generate SVG for a component in an unknown ECAD tool format({}).".format(
                     tool
@@ -1078,8 +1067,7 @@ class Part(SkidlBaseObject):
         try:
             gen_func = getattr(self, "_gen_pinboxes_{}".format(tool))
         except AttributeError:
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't generate pinboxes for a component in an unknown ECAD tool format({}).".format(
                     tool
@@ -1150,8 +1138,7 @@ class Part(SkidlBaseObject):
         # Update the part's tag.
         str_tag = str(value)
         if re.compile(r"[^a-zA-Z0-9\-_]").search(str_tag):
-            log_and_raise(
-                logger,
+            active_logger.raise_(
                 ValueError,
                 "Can't set part tag to {} it contains disallowed characters.".format(
                     str_tag
@@ -1268,6 +1255,11 @@ class Part(SkidlBaseObject):
 
 ##############################################################################
 
+"""Shortcut for creating a Part template."""
+PartTmplt = functools.partial(Part, dest=TEMPLATE)
+
+##############################################################################
+
 
 class SkidlPart(Part):
     """
@@ -1279,17 +1271,13 @@ class SkidlPart(Part):
         a part and then add pins to it without it being added to the netlist.
     """
 
-    from .defines import SKIDL, TEMPLATE
-
     def __init__(
-        self,
-        lib=None,
-        name=None,
-        dest=TEMPLATE,
-        tool=SKIDL,
-        connections=None,
-        **attribs
+        self, lib=None, name=None, dest=TEMPLATE, tool=None, connections=None, **attribs
     ):
+        from .tools import SKIDL
+
+        if not tool:
+            tool = SKIDL
         super().__init__(lib, name, dest, tool, connections, attribs)
 
 
@@ -1377,7 +1365,7 @@ class PartUnit(Part):
         self.pins = list(set(self.pins + new_pins))
 
     def validate(self):
-        """ Check that unit pins point to the parent part. """
+        """Check that unit pins point to the parent part."""
         for pin in self.pins:
             assert id(pin.part) == id(self.parent)
 
