@@ -1,32 +1,17 @@
 # -*- coding: utf-8 -*-
 
-# MIT license
-#
-# Copyright (C) 2016 by XESS Corp.
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software withcurrent_level restriction, including withcurrent_level limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHcurrent_level WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# current_level OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
 
 """
 Utility functions used by the rest of SKiDL.
 """
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import (  # isort:skip
+    absolute_import,
+    division,
+    print_function,
+    unicode_literals,
+)
 
 import collections
 import os
@@ -41,7 +26,6 @@ from contextlib import contextmanager
 from future import standard_library
 
 from .common import *
-from .defines import *
 
 standard_library.install_aliases()
 
@@ -100,7 +84,7 @@ def is_binary_file(filename):
 
 
 def merge_dicts(dct, merge_dct):
-    """ 
+    """
     Dict merge that recurses through both dicts and updates keys.
 
     Args:
@@ -139,7 +123,7 @@ def find_and_open_file(
                  subdirectories withcurrent_level limit.
     """
 
-    from .logger import logger
+    from .logger import active_logger
 
     if os.path.isabs(filename):
         # Ignore search paths if the file already has an absolute path.
@@ -187,19 +171,19 @@ def find_and_open_file(
     if allow_failure:
         return None, None
     else:
-        log_and_raise(
-            logger, FileNotFoundError, "Can't open file: {}.\n".format(filename)
+        active_logger.raise_(
+            FileNotFoundError, "Can't open file: {}.\n".format(filename)
         )
 
 
 def add_unique_attr(obj, name, value, check_dup=False):
     """Create an attribute if the attribute name isn't already used."""
-    from .logger import logger
+    from .logger import active_logger
 
     try:
         getattr(obj, name)
         if check_dup:
-            logger.warn(
+            active_logger.warning(
                 "Unable to create attribute {name} of type {typ1} because one already exists of type {typ2} in {obj}".format(
                     name=name,
                     typ1=type(value),
@@ -226,6 +210,7 @@ def num_to_chars(num):
 
 def rmv_quotes(s):
     """Remove starting and ending quotes from a string."""
+
     if not isinstance(s, basestring):
         return s
 
@@ -241,6 +226,7 @@ def rmv_quotes(s):
 
 def add_quotes(s):
     """Return string with added quotes if it contains whitespace or parens."""
+
     if not isinstance(s, basestring):
         return s
 
@@ -268,7 +254,7 @@ def is_iterable(x):
 
 def to_list(x):
     """
-    Return x if it is already a list, or return a list if x is a scalar.
+    Return x if it is already a list, or return a list containing x if x is a scalar.
     """
     if isinstance(x, (list, tuple)):
         return x  # Already a list, so just return it.
@@ -315,21 +301,41 @@ def flatten(nested_list):
     return lst
 
 
+def set_attr(objs, attr, value):
+    """Remove an attribute from a list of objects."""
+    try:
+        for o in objs:
+            setattr(o, attr, value)
+    except TypeError:
+        setattr(obj, attr, value)
+
+
 def rmv_attr(objs, attr):
     """Remove an attribute from a list of objects."""
-    for o in to_list(objs):
-        try:
+    try:
+        for o in objs:
             delattr(o, attr)
-        except AttributeError:
-            pass
+    except TypeError:
+        delattr(obj, attr)
 
 
 def from_iadd(objs):
     """Return True if one or more objects have attribute iadd_flag set to True."""
-    return any([getattr(e, "iadd_flag", False) for e in to_list(objs)])
+    try:
+        for o in objs:
+            if getattr(o, "iadd_flag", False):
+                return True
+        return False
+    except TypeError:
+        return getattr(objs, "iadd_flag", False)
 
 
-def clr_iadd(objs):
+def set_iadd(objs, value):
+    """Set iadd_flag with T/F value for a list of objects."""
+    set_attr(objs, "iadd_flag", value)
+
+
+def rmv_iadd(objs):
     """Delete iadd_flag attribute from a list of objects."""
     rmv_attr(objs, "iadd_flag")
 
@@ -558,7 +564,7 @@ def expand_indices(slice_min, slice_max, match_regex, *indices):
     """
     Expand a list of indices into a list of integers and strings.
 
-    This function takes the indices used to select pins of parts and 
+    This function takes the indices used to select pins of parts and
     lines of buses and returns a flat list of numbers and strings.
     String and integer indices are put in the list unchanged, but
     slices are expanded into a list of integers before entering the
@@ -567,14 +573,14 @@ def expand_indices(slice_min, slice_max, match_regex, *indices):
     Args:
         slice_min: The minimum possible index.
         slice_max: The maximum possible index (used for slice indices).
-        match_regex: If true, 
+        match_regex: If true,
         indices: A list of indices made up of numbers, slices, text strings.
 
     Returns:
         A linear list of all the indices made up only of numbers and strings.
     """
 
-    from .logger import logger
+    from .logger import active_logger
 
     def expand_slice(slc):
         """Expand slice notation."""
@@ -587,8 +593,7 @@ def expand_indices(slice_min, slice_max, match_regex, *indices):
         # Do this if it's a downward slice (e.g., [7:0]).
         if start > stop:
             if slc.start and slc.start > slice_max:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     IndexError,
                     "Index current_level of range ({} > {})!".format(
                         slc.start, slice_max
@@ -601,8 +606,7 @@ def expand_indices(slice_min, slice_max, match_regex, *indices):
         # Do this if it's a normal (i.e., upward) slice (e.g., [0:7]).
         else:
             if slc.stop and slc.stop > slice_max:
-                log_and_raise(
-                    logger,
+                active_logger.raise_(
                     IndexError,
                     "Index current_level of range ({} > {})!".format(
                         slc.stop, slice_max
@@ -684,8 +688,8 @@ def expand_indices(slice_min, slice_max, match_regex, *indices):
                 # added to the list.
                 ids.extend(explode(id.strip()))
         else:
-            log_and_raise(
-                logger, TypeError, "Unknown type in index: {}.".format(type(indx))
+            active_logger.raise_(
+                TypeError, "Unknown type in index: {}.".format(type(indx))
             )
 
     # Return the completely expanded list of indices.
@@ -711,7 +715,7 @@ def find_num_copies(**attribs):
         or lists/tuples of the same length.)
     """
 
-    from .logger import logger
+    from .logger import active_logger
 
     num_copies = set()
     for k, v in list(attribs.items()):
@@ -722,14 +726,12 @@ def find_num_copies(**attribs):
 
     num_copies = list(num_copies)
     if len(num_copies) > 2:
-        log_and_raise(
-            logger,
+        active_logger.raise_(
             ValueError,
             "Mismatched lengths of attributes: {}!".format(num_copies),
         )
     elif len(num_copies) > 1 and min(num_copies) > 1:
-        log_and_raise(
-            logger,
+        active_logger.raise_(
             ValueError,
             "Mismatched lengths of attributes: {}!".format(num_copies),
         )
@@ -750,6 +752,7 @@ def opened(f_or_fn, mode):
            object, or a filename to open.
        mode: The mode to open the file in.
     """
+
     if isinstance(f_or_fn, basestring):
         with open(f_or_fn, mode, encoding="utf-8") as f:
             yield f
@@ -782,11 +785,6 @@ def expand_buses(pins_nets_buses):
     return pins_nets
 
 
-def log_and_raise(logger_in, exc_class, message):
-    logger_in.error(message)
-    raise exc_class(message)
-
-
 # Regular expression for parsing nested S-expressions.
 _sexp_term_regex = re.compile(
     r"""(?mx)
