@@ -1,6 +1,6 @@
 from skidl import *
 
-opamp = Part(lib="Amplifier_Operational.lib", name="AD8676xR", symtx="V")
+opamp = Part(lib="Amplifier_Operational.lib", name="AD8676xR", symtx="H")
 opamp.uA.p2 += Net("IN1")
 opamp.uA.p3 += Net("IN2")
 opamp.uA.p1 += Net("OUT")
