@@ -13,9 +13,13 @@ gnd[1] += opamp[4]
 
 r = Part("Device", "R_US", dest=TEMPLATE, tx_ops="L")
 
-Net("IN") & r(value="4K7", symtx="L") & opamp.uA[2] & r(
-    value="4K7", symtx="L"
-) & opamp.uA[1]
+(
+    Net("IN")
+    & r(value="4K7", symtx="L")
+    & opamp.uA[2]
+    & r(value="4K7", symtx="L")
+    & opamp.uA[1]
+)
 gnd[1] += opamp.uA[3]
 
 opamp.uA[1] & r(value="10K") & gnd[1]
