@@ -1,6 +1,6 @@
 from skidl import *
 
-lib_search_paths[KICAD].append("/home/devb/xesscorp/KiCad/libraries")
+lib_search_paths[KICAD].append("/home/devb/tech_stuff/KiCad/libraries")
 uc = Part(lib="wch.lib", name="CH551G", dest=TEMPLATE)
 uc.split_pin_names("/")
 usb = Part(lib="Connector.lib", name="USB_B_Micro", symtx="H")
@@ -13,5 +13,7 @@ uc_spare["UDP"] & uc_spare["UDM"]
 
 stubs = uc1["UDM"].get_nets()
 stubs.extend(uc1["UDP"].get_nets())
+for s in stubs:
+    s.stub = True
 
-generate_svg(net_stubs=stubs)
+generate_svg()
