@@ -9,9 +9,22 @@ vcct = Part("power", "VCC")
 
 gnd = Net("GND")
 vcc = Net("VCC")
-gnd & gndt & r() & r() & (
-    r(symtx="l") | r(symtx="R")
-) & r() & r() & r() & r() & r() & r() & r() & vcct & vcc
+(
+    gnd
+    & gndt
+    & r()
+    & r()
+    & (r(symtx="l") | r(symtx="R"))
+    & r()
+    & r()
+    & r()
+    & r()
+    & r()
+    & r()
+    & r()
+    & vcct
+    & vcc
+)
 
 generate_svg()
 sys.exit()
