@@ -1,3 +1,7 @@
+# -*- coding: utf-8 -*-
+
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
+
 import os
 
 from skidl import *
@@ -14,11 +18,14 @@ def setup_function(f):
     global files_at_start
     files_at_start = set(os.listdir(os.getcwd()))
 
-    default_circuit.reset()
+    # Mini-reset to remove circuitry but retain any loaded libraries.
+    default_circuit.mini_reset()
 
     lib_search_paths.clear()
     lib_search_paths.update({tool: [os.getcwd(), this_file_dir] for tool in ALL_TOOLS})
-    lib_search_paths.update({SKIDL: [os.getcwd(), this_file_dir, get_filename("../skidl/libs")]})
+    lib_search_paths.update(
+        {SKIDL: [os.getcwd(), this_file_dir, get_filename("../skidl/libs")]}
+    )
 
     set_default_tool(KICAD)
     set_query_backup_lib(INITIAL_QUERY_BACKUP_LIB)
