@@ -1,8 +1,25 @@
+# -*- coding: utf-8 -*-
+
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
+
 import pytest
 
-from skidl import *
+from skidl import (
+    ERC,
+    SKIDL,
+    TEMPLATE,
+    Bus,
+    Circuit,
+    Net,
+    Part,
+    Pin,
+    generate_netlist,
+    generate_xml,
+    subcircuit,
+    super,
+)
 
-from .setup_teardown import *
+from .setup_teardown import setup_function, teardown_function
 
 
 def test_subcircuit_1():
