@@ -1,10 +1,12 @@
-import logging
+# -*- coding: utf-8 -*-
+
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
 
 import pytest
 
-from skidl import *
+from skidl import Part
 
-from .setup_teardown import *
+from .setup_teardown import setup_function, teardown_function
 
 
 def test_part_unit_1():
