@@ -1,8 +1,12 @@
+# -*- coding: utf-8 -*-
+
+# The MIT License (MIT) - Copyright (c) 2016-2021 Dave Vandenbout.
+
 import pytest
 
-from skidl import *
+from skidl import TEMPLATE, Bus, Net, Part, Pin, ProtoNet, package
 
-from .setup_teardown import *
+from .setup_teardown import setup_function, teardown_function
 
 
 def test_protonet_1():
